@@ -7,7 +7,7 @@ use std::fmt;
 use indenter::indented;
 
 use crate::chain::Chain;
-use crate::protocol::{Diagnostic, DiagnosticSnippet, DiagnosticReporter, Severity};
+use crate::protocol::{Diagnostic, DiagnosticReporter, DiagnosticSnippet, Severity};
 
 /**
 Reference implementation of the [DiagnosticReporter] trait. This is generally
@@ -17,7 +17,11 @@ you want custom reporting for your tool or app.
 pub struct MietteReporter;
 
 impl MietteReporter {
-    fn render_snippet(&self, f: &mut fmt::Formatter<'_>, snippet: &DiagnosticSnippet) -> fmt::Result {
+    fn render_snippet(
+        &self,
+        f: &mut fmt::Formatter<'_>,
+        snippet: &DiagnosticSnippet,
+    ) -> fmt::Result {
         use fmt::Write as _;
         write!(f, "\n[{}]", snippet.source_name)?;
         if let Some(msg) = &snippet.message {
@@ -103,9 +107,9 @@ impl DiagnosticReporter for MietteReporter {
         }
 
         let sev = match diagnostic.severity() {
-            Severity::Error => "Error",
-            Severity::Warning => "Warning",
-            Severity::Advice => "Advice",
+            Some(Severity::Error) | None => "Error",
+            Some(Severity::Warning) => "Warning",
+            Some(Severity::Advice) => "Advice",
         };
         write!(f, "{}[{}]: {}", sev, diagnostic.code(), diagnostic)?;
 
@@ -126,15 +130,12 @@ impl DiagnosticReporter for MietteReporter {
         if let Some(snippets) = diagnostic.snippets() {
             writeln!(f)?;
             for snippet in snippets {
-                self.render_snippet(f, snippet)?;
+                self.render_snippet(f, &snippet)?;
             }
         }
 
         if let Some(help) = diagnostic.help() {
-            writeln!(f)?;
-            for msg in help {
-                writeln!(f, "﹦{}", msg)?;
-            }
+            writeln!(f,  "﹦{}", help)?;
         }
 
         Ok(())
@@ -151,9 +152,9 @@ impl DiagnosticReporter for JokeReporter {
         }
 
         let sev = match diagnostic.severity() {
-            Severity::Error => "error",
-            Severity::Warning => "warning",
-            Severity::Advice => "advice",
+            Some(Severity::Error) | None => "error",
+            Some(Severity::Warning) => "warning",
+            Some(Severity::Advice) => "advice",
         };
         writeln!(
             f,
@@ -162,16 +163,14 @@ impl DiagnosticReporter for JokeReporter {
             diagnostic,
             diagnostic
                 .help()
-                .unwrap_or_else(|| Box::new(vec!["have you tried not failing?"].into_iter()))
-                .collect::<Vec<&str>>()
-                .join(" ")
+                .unwrap_or(&"have you tried not failing?")
         )?;
         writeln!(
             f,
             "miette, her eyes enormous: you {} miette? you {}? oh! oh! jail for mother! jail for mother for One Thousand Years!!!!",
             diagnostic.code(),
             diagnostic.snippets().map(|snippets| {
-                snippets.iter().map(|snippet| snippet.message.clone()).collect::<Option<Vec<String>>>()
+                snippets.map(|snippet| snippet.message).collect::<Option<Vec<String>>>()
             }).flatten().map(|x| x.join(", ")).unwrap_or_else(||"try and cause miette to panic".into())
         )?;
 
