@@ -1,7 +1,8 @@
-use std::fmt;
+use std::{fmt, sync::Arc};
 
 use miette::{
-    Diagnostic, DiagnosticSnippet, DiagnosticReporter, MietteError, MietteReporter, Severity, SourceSpan,
+    Diagnostic, DiagnosticReporter, DiagnosticSnippet, MietteError, MietteReporter,
+    SourceSpan,
 };
 use thiserror::Error;
 
@@ -22,16 +23,12 @@ impl Diagnostic for MyBad {
         &"oops::my::bad"
     }
 
-    fn severity(&self) -> Severity {
-        Severity::Error
+    fn help(&self) -> Option<&(dyn std::fmt::Display)> {
+        Some(&"try doing it better next time?")
     }
 
-    fn help(&self) -> Option<Box<dyn '_ + Iterator<Item = &'_ str>>> {
-        Some(Box::new(vec!["try doing it better next time?"].into_iter()))
-    }
-
-    fn snippets(&self) -> Option<&[DiagnosticSnippet]> {
-        Some(&self.snippets)
+    fn snippets(&self) -> Option<Box<dyn Iterator<Item = DiagnosticSnippet>>> {
+        Some(Box::new(self.snippets.clone().into_iter()))
     }
 }
 
@@ -42,7 +39,7 @@ fn basic() -> Result<(), MietteError> {
     };
     let out = format!("{:?}", err);
     assert_eq!(
-        "Error[oops::my::bad]: oops!\n\n﹦try doing it better next time?\n".to_string(),
+        "Error[oops::my::bad]: oops!\n﹦try doing it better next time?\n".to_string(),
         out
     );
     Ok(())
@@ -51,25 +48,27 @@ fn basic() -> Result<(), MietteError> {
 #[test]
 fn fancy() -> Result<(), MietteError> {
     let src = "source\n  text\n    here".to_string();
+    let len = src.len();
     let err = MyBad {
         snippets: vec![DiagnosticSnippet {
             message: Some("This is the part that broke".into()),
             source_name: "bad_file.rs".into(),
-            source: Box::new(src.clone()),
-            highlights: Some(vec![
-                ("this bit here".into(), SourceSpan {
+            source: Arc::new(src),
+            highlights: Some(vec![(
+                "this bit here".into(),
+                SourceSpan {
                     start: 9.into(),
                     end: 12.into(),
-                })
-            ]),
+                },
+            )]),
             context: SourceSpan {
                 start: 0.into(),
-                end: (src.len() - 1).into(),
+                end: (len - 1).into(),
             },
         }],
     };
     let out = format!("{:?}", err);
     // println!("{}", out);
-    assert_eq!("Error[oops::my::bad]: oops!\n\n[bad_file.rs] This is the part that broke:\n\n    1  | source\n    2  |   text\n    ⫶  |   ^^^^ this bit here\n    3  |     here\n\n﹦try doing it better next time?\n".to_string(), out);
+    assert_eq!("Error[oops::my::bad]: oops!\n\n[bad_file.rs] This is the part that broke:\n\n    1  | source\n    2  |   text\n    ⫶  |   ^^^^ this bit here\n    3  |     here\n﹦try doing it better next time?\n".to_string(), out);
     Ok(())
 }
